﻿
Public Class NXT

    Private Sub BuscarPuertos_Click(sender As Object, e As EventArgs) Handles BuscarPuertos.Click 'Función para buscar puertos serie activados.
        ComboPuertos.Items.Clear()

        For Each PuertoDisponible As String In My.Computer.Ports.SerialPortNames 'Busca cada uno de los puertos serie disponibles en el ordenador
            ComboPuertos.Items.Add(PuertoDisponible)
        Next
        If ComboPuertos.Items.Count > 0 Then                   ' si hay puertos serie utilizandose los muestra en el combobox
            ComboPuertos.Text = ComboPuertos.Items(0)
        Else
            MessageBox.Show("NINGUN PUERTO ENCONTRADO")     ' si no hay ninguno disponible te indica que no se ha encontrado ningún puerto
            ComboPuertos.Items.Clear()
        End If
    End Sub

    Private Sub Conexion_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Conexion.Click 'Función para establecer la conexión con un puerto serie.

        Dim byteOut(15) As Byte   ' Variable que almacena la trama para enviar a través del puerto.

        Try
            If Conexion.Text = "Conectar" Then       'Si pulsamos conectar.

                With SerialPort    ' Definir los parametros de la conexión del puerto serie.

                    .PortName = ComboPuertos.Text
                    .BaudRate = 9600
                    .Parity = IO.Ports.Parity.None
                    .DataBits = 8
                    .StopBits = IO.Ports.StopBits.One
                    .ReadTimeout = 300
                    .WriteTimeout = 300

                End With

                SerialPort.Open()                       'Abre el puerto serie elegido.
                EstadoConexion.Text = "Conectado"       'Emitimos el mensaje de conexión.
                EstadoConexion.BackColor = Drawing.Color.LawnGreen
                Velocidad.Text = "25"                   'Se establece una velocidad por defecto al iniciar la conexión.
                Conexion.Text = "Desconectar"           'El pulsador para conectar ahora sirve para desconectar.

                TimerBatery.Enabled = True              'Se activa el timer para la pedida del nivel de batería.
                Call Bateria_Click(sender, e)           'Llamamos a la funcion de la batería para pedir nada mas iniciar el nivel de bateria del NXT.

            ElseIf Conexion.Text = "Desconectar" Then   'Si pulsamos desconectar.

                SerialPort.Close()                      'Cerramos el puerto.
                EstadoConexion.Text = "Desconectado"    'Emitimimos el mensaje de Desconexión.
                EstadoConexion.BackColor = Drawing.Color.Red
                Conexion.Text = "Conectar"              'El pulsador vuelve a tener la función de conectar.

                TimerBatery.Enabled = False             'Desactivamos el timer de la bateria para que no se envie ninguna trama mientras el puerto esta cerrado.
                MarcadorBateria.BackColor = Drawing.Color.LightSteelBlue  'Reiniciamos el marcador de la batería.
                MarcadorBateria.Text = ""
                Velocidad.Text = "0"                    'Eliminamos la velocidad por defecto.
            End If

        Catch ex As Exception
            MsgBox("NO SE PUDO CONECTAR CON EL PUERTO " & CStr(SerialPort.PortName)) 'Si no se puede conectar con el puerto elegido nos llega un mensaje de error.
        End Try

    End Sub


    Private Sub Bateria_Click(sender As Object, e As EventArgs) Handles Bateria.Click 'Función para enviar la trama para pedir el nivel de bateria del NXT y devolver el valor.

        Dim byteOut(3) As Byte      'Variable para almacenar la trama a enviar.
        Dim byteIn(6) As Byte       'Variable que almacena la trama recibida.
        Dim Voltage, t As Integer
        t = 0

        Try
            byteOut(0) = &H2  '2 bytes en mensaje de salida.
            byteOut(1) = &H0  '0 para NXT
            byteOut(2) = &H0  '&H0 = esperamos respuesta; &H80 = no esperamos respuesta.
            byteOut(3) = &HB  '$HB = Comando de lectura de batería.
            SerialPort.Write(byteOut, 0, 4) 'Trama enviada por el puerto serie.

            While SerialPort.BytesToRead = 0 'Bucle que espera a que se reciban datos.

            End While

            byteIn(0) = SerialPort.ReadByte 'Numero de bytes en el mensaje recibido.
            byteIn(1) = SerialPort.ReadByte 'Debe ser cero para el NXT.
            For t = 2 To 1 + byteIn(0)      'Leemos el resto del mensaje y lo almacenamos en nuestra variable de recepción.
                byteIn(t) = SerialPort.ReadByte()
            Next
            Voltage = byteIn(5) + byteIn(6) * 256 'El voltaje tiene el byte bajo en el 5 y el byte alto en el 6.
            MarcadorBateria.Text = Voltage / 1000 'Muestra el voltaje.

            If MarcadorBateria.Text > 5 Then  'Cambiamos el color del display en función del voltaje.
                MarcadorBateria.BackColor = Drawing.Color.LawnGreen
            ElseIf MarcadorBateria.Text < 5 And MarcadorBateria.Text > 2 Then
                MarcadorBateria.BackColor = Drawing.Color.Orange
            ElseIf MarcadorBateria.Text < 2 Then
                MarcadorBateria.BackColor = Drawing.Color.Red
            End If

        Catch ex As Exception
            TimerBatery.Enabled = False
            MsgBox("ERROR EN LA LECTURA DEL NIVEL DE BATERIA") 'Muestra en pantalla un mensaje de Error.
        End Try

    End Sub

    Private Sub TimerBatery_Tick(sender As Object, e As EventArgs) Handles TimerBatery.Tick
        Call Bateria_Click(sender, e)   'Este Timer esta establecido para que llame a la función de lectura de batería cada 10s.
    End Sub

    Private Sub Alarma_Click(sender As Object, e As EventArgs) Handles Alarma.Click

        Dim byteOut(7) As Byte 'Variable que almacena la trama a enviar.
        Dim freq As Int16      'Variable de Frecuencia.
        Dim i As Integer = 0   'Contador
        Dim f As Integer = 0   'Contador

        Try
            byteOut(0) = &H6  '6 bytes in output message
            byteOut(1) = &H0  'should be 0 for NXT
            byteOut(2) = &H80 '&H0 = reply expected &H80 = no reply expected
            byteOut(3) = &H3  'PLAYTONE
            freq = Frecuencia.Value

            Do Until i = 100 Or f = 2
                byteOut(4) = freq - (freq / 256) * 256 'low byte of freq
                byteOut(5) = freq / 256 'high byte of freq
                byteOut(6) = &H64 'low byte of duration = 100 ms
                byteOut(7) = &H0  'high byte of duration
                SerialPort.Write(byteOut, 0, 8)
                i = i + 1
                If i = 100 Then
                    i = 0
                    Do Until i = 100
                        byteOut(4) = (freq / 2) - ((freq / 2) / 256) * 256
                        byteOut(5) = (freq / 2) / 256
                        byteOut(6) = &HC8
                        SerialPort.Write(byteOut, 0, 8)
                        i = i + 1
                    Loop
                    i = 0
                    f = f + 1
                End If
            Loop

        Catch ex As Exception
            MsgBox(ex.ToString)
        End Try

    End Sub


    Public Sub Adelante_MouseDown(sender As Object, e As EventArgs) Handles Adelante.MouseDown 'Función que envia la trama para que el NXT avance.

        Dim byteOut(13) As Byte ' Variable que almacena la trama a enviar.

        Try
            byteOut(0) = &HC  'Cantidad de bytes ha enviar
            byteOut(1) = &H0  'Debe ser 0 para el NXT.
            byteOut(2) = &H80 '&H0 = Si queremos respuesta, &H80 = Si no queremos respuesta.
            byteOut(3) = &H4  'Tipo de comando que indica que se van a controlar los motores.
            byteOut(4) = &H1  'Puerto de motores con el que se va a interactuar. En este caso Puerto B.
            byteOut(5) = CByte(Velocidad.Text) ' Velocidad de salida en un rango de 0 a 100.
            byteOut(6) = &H5  'Modo Motor ON con regulación y sincronización entre los dos motores.
            byteOut(7) = &H1  'Ajusta la velociad en funcion de la carga.
            byteOut(8) = &H0  'Sincronización del radio de giro, en este caso no es necesario.
            byteOut(9) = &H20 'Activa la alimentación del motor en el puerto especificado.
            byteOut(10) = &H0 'Los siguientes 4 bytes son a 0.
            byteOut(11) = &H0
            byteOut(12) = &H0
            byteOut(13) = &H0
            SerialPort.Write(byteOut, 0, 14) ' Envio los bytes del comando a traves del puerto serie.

            byteOut(4) = &H2 'Puerto de motores a interactuar, puerto C.
            SerialPort.Write(byteOut, 0, 14) 'Como el resto de bytes se mantienen iguales vuelvo a enviar el comando.

        Catch ex As Exception
            MsgBox("LOS MOTORES NO RESPONDEN") 'En caso de error muestro un mensaje en pantalla.
        End Try

    End Sub

    Private Sub Adelante_MouseUp(sender As Object, e As MouseEventArgs) Handles Adelante.MouseUp
        Call Parada_Click(sender, e) 'Al dejar de mantener pulsado llama a la subrutina de parada.
    End Sub

    Private Sub Atras_MouseDown(sender As Object, e As EventArgs) Handles Atras.MouseDown

        Dim byteOut(13) As Byte

        Try
            byteOut(0) = &HC
            byteOut(1) = &H0
            byteOut(2) = &H80
            byteOut(3) = &H4
            byteOut(4) = &H1
            byteOut(5) = CByte(256 - Velocidad.Text)
            byteOut(6) = &H5
            byteOut(7) = &H1
            byteOut(8) = &H0
            byteOut(9) = &H20
            byteOut(10) = &H0
            byteOut(11) = &H0
            byteOut(12) = &H0
            byteOut(13) = &H0
            SerialPort.Write(byteOut, 0, 14)

            byteOut(4) = &H2 'cambiamos el byte del motor el resto siguen iguales
            SerialPort.Write(byteOut, 0, 14)

        Catch ex As Exception
            MsgBox("LOS MOTORES NO RESPONDEN")
        End Try

    End Sub

    Private Sub Atras_MouseUp(sender As Object, e As MouseEventArgs) Handles Atras.MouseUp
        Call Parada_Click(sender, e)
    End Sub

    Public Sub Izquierda_MouseDown(sender As Object, e As EventArgs) Handles Izquierda.MouseDown 'Función que envia la trama para que el NXT gire a la izquierda

        Dim byteOut(13) As Byte

        Try
            byteOut(0) = &HC
            byteOut(1) = &H0
            byteOut(2) = &H80
            byteOut(3) = &H4
            byteOut(4) = &H1
            byteOut(5) = CByte((Velocidad.Text) / 2) 'Dividimos la velocidad entre dos para que el giro sea menos brusco
            byteOut(6) = &H7                        'Regulacion con exactitud de la salida del motor
            byteOut(7) = &H1
            byteOut(8) = &H0
            byteOut(9) = &H20
            byteOut(10) = &H0
            byteOut(11) = &H0
            byteOut(12) = &H0
            byteOut(13) = &H0
            SerialPort.Write(byteOut, 0, 14)

            byteOut(4) = &H2
            byteOut(5) = CByte(256 - (Velocidad.Text) / 2)
            SerialPort.Write(byteOut, 0, 14)

        Catch ex As Exception
            MsgBox("LOS MOTORES NO RESPONDEN")
        End Try

    End Sub

    Private Sub Izquierda_MouseUp(sender As Object, e As MouseEventArgs) Handles Izquierda.MouseUp
        Call Parada_Click(sender, e)
    End Sub

    Public Sub Derecha_MouseDown(sender As Object, e As EventArgs) Handles Derecha.MouseDown

        Dim byteOut(13) As Byte

        Try
            byteOut(0) = &HC
            byteOut(1) = &H0
            byteOut(2) = &H80
            byteOut(3) = &H4
            byteOut(4) = &H2
            byteOut(5) = CByte((Velocidad.Text) / 2)
            byteOut(6) = &H7
            byteOut(7) = &H1
            byteOut(8) = &H0
            byteOut(9) = &H20
            byteOut(10) = &H0
            byteOut(11) = &H0
            byteOut(12) = &H0
            byteOut(13) = &H0
            SerialPort.Write(byteOut, 0, 14)

            byteOut(4) = &H1
            byteOut(5) = CByte(256 - (Velocidad.Text) / 2)
            SerialPort.Write(byteOut, 0, 14)

        Catch ex As Exception
            MsgBox("LOS MOTORES NO RESPONDEN")
        End Try

    End Sub

    Private Sub Derecha_MouseUp(sender As Object, e As MouseEventArgs) Handles Derecha.MouseUp
        Call Parada_Click(sender, e)
    End Sub

    Public Sub Parada_Click(sender As Object, e As EventArgs) Handles Parada.Click

        Dim byteOut(13) As Byte

        Try
            byteOut(0) = &HC  'Cantidad de bytes ha enviar
            byteOut(1) = &H0  '0 para el NXT.
            byteOut(2) = &H80 '&H0 = Si queremos respuesta, &H80 = Si no queremos respuesta.
            byteOut(3) = &H4  'Tipo de comando que indica que se van a controlar los motores.
            byteOut(4) = &H1  'Puerto de motores con el que se va a interactuar. En este caso Puerto B.
            byteOut(5) = &H0  'Velocidad de salida en un rango de 0 a 100.
            byteOut(6) = &H0  'Modo freno.
            byteOut(7) = &H1  'Ajusta la velociad en funcion de la carga.
            byteOut(8) = &H0  'Sincronización del radio de giro, en este caso no es necesario.
            byteOut(9) = &H0  'Desactiva la alimentación del motor en el puerto especificado.
            byteOut(10) = &H0 'Los siguientes 4 bytes son a 0.
            byteOut(11) = &H0
            byteOut(12) = &H0
            byteOut(13) = &H0
            SerialPort.Write(byteOut, 0, 14) 'Envio los bytes del comando a traves del puerto serie.

            byteOut(4) = &H2  'Puerto de motores a interactuar, puerto C.
            SerialPort.Write(byteOut, 0, 14)  'Envio los bytes del comando a traves del puerto serie.

        Catch ex As Exception
            MsgBox("ERROR STOP")
        End Try

    End Sub


    Private Sub Medicionultrasonido_Click(sender As Object, e As EventArgs) Handles Medicionultrasonido.Click 'Función que envia las tramas encargadas de activar el sensor de ultrasonidos y leer la distancia.

        Dim byteOut(8) As Byte
        Dim byteIn(22) As Byte
        Dim Distancia, g As Integer
        g = 0

        Try

            'SET INPUT MODE Establecemos el modo de funcionamiento del sensor de ultrasonidos.
            byteOut(0) = &H5
            byteOut(1) = &H0
            byteOut(2) = &H80
            byteOut(3) = &H5  'Comando a enviar (SETINPUTMODE)
            byteOut(4) = &H0  'Puerto en el que estaria conectado el sensor (0 = puerto 1 del NXT)
            byteOut(5) = &HB  'Tipo de sensor (LOWSPEED_9V)
            byteOut(6) = &H0  'Modo del sensor (RAWMODE)
            SerialPort.Write(byteOut, 0, 7)

            'LS WRITE   Indicamos al sensor conectado en el puerto 1 que queremos leer 2 bytes en el registro 0x42.
            byteOut(0) = &H7
            byteOut(1) = &H0
            byteOut(2) = &H80
            byteOut(3) = &HF 'Comando a enviar (LSWRITE)
            byteOut(4) = &H0 'Puerto en el que estaria conectado el sensor (0 = puerto 1 del NXT)
            byteOut(5) = &H2 'Longitud de bytes que se quieren escribir en el bus I2C.
            byteOut(6) = &H2 'Longitud de bytes que se quieren recibir por el bus I2C.
            byteOut(7) = &H2 'Direccion del dispositivo I2C, para el NXT es 2.
            byteOut(8) = &H42 'Direccion del registro donde se guarda la lectura en el NXT.
            SerialPort.Write(byteOut, 0, 9)

            'LS GET STATUS  Esperamos hasta que haya 1 byte disponible para leer.
            byteOut(0) = &H3
            byteOut(1) = &H0
            byteOut(2) = &H0 'Si queremos respuesta
            byteOut(3) = &HE 'Comando LSGETSTATUS
            byteOut(4) = &H0 'Puerto 1
            SerialPort.Write(byteOut, 0, 5)

            byteIn(0) = SerialPort.ReadByte 'Número de bytes a recibir
            byteIn(1) = SerialPort.ReadByte '0 para NXT
            For g = 2 To 1 + byteIn(0) 'leemos el resto del mensaje
                byteIn(g) = SerialPort.ReadByte()
            Next


            'LS READ Realizamos la lectura del registro.
            byteOut(0) = &H3
            byteOut(1) = &H0
            byteOut(2) = &H0 'indicamos que queremos respuesta
            byteOut(3) = &H10 'Comando LSREAD
            byteOut(4) = &H0  'Puerto 1
            SerialPort.Write(byteOut, 0, 5)

            byteIn(0) = SerialPort.ReadByte 'Número de bytes a recibir
            byteIn(1) = SerialPort.ReadByte '0 para NXTXT
            For g = 2 To 1 + byteIn(0) 'leemos el resto del mensaje
                byteIn(g) = SerialPort.ReadByte()
            Next

            Distancia = byteIn(6)   'La distancia recogida por el sensor se encuentra en el byte 6
            Medida.Text = Distancia 'Mostramos la distancia en el display

            If Distancia < 30 Then   'Si tenemos en marcha el robot a la vez que esta leyendo la distancia, el robot se detendrá si la distancia con un obstaculo es menor a 20cm.

                Call Parada_Click(sender, e) 'Llamamos a la funcion de Parada.

            End If

            TimerUltraSonic.Enabled = True 'Activamos el timer del ultrasonidos que preguntará por la distancia cada 300ms

        Catch ex As Exception
            MsgBox("ERROR EN LA MEDICION") 'Mensaje a lanzar en caso de error.
        End Try

    End Sub

    Private Sub ParadaUltrasonido_Click(sender As Object, e As EventArgs) Handles ParadaUltrasonido.Click 'Se detiene la lectura de distancia a través del sensor US.
        TimerUltraSonic.Enabled = False  'Detenemos el timer.
        Medida.Text = ""                 'Ponemos el display a 0.
    End Sub

    Private Sub TimerUltraSonic_Tick(sender As Object, e As EventArgs) Handles TimerUltraSonic.Tick 'Timer del US, Llama a la función de lectura por US cada 300ms.
        Call Medicionultrasonido_Click(sender, e)
    End Sub

    Private Sub SensorLuz_Click(sender As Object, e As EventArgs) Handles SensorLuz.Click

        Dim byteOut(6) As Byte
        Dim byteIn(17) As Byte
        Dim g As Integer = 0
        TimerColor.Enabled = True 'Activamos el timer que llama a la funcion de lectura de color cada 300ms.

        Try
            'SET INPUT MODE
            byteOut(0) = &H5 ' Cantidad de bytes que va a enviar 
            byteOut(1) = &H0 ' Siempre 0
            byteOut(2) = &H80 ' &H0 = se espera respuesta &H80 = no se espera respuesta
            byteOut(3) = &H5 ' Comando a enviar (SETINPUTMODE)
            byteOut(4) = &H2 'Puerto
            byteOut(5) = &HD 'tipo de sensor
            byteOut(6) = &H0 'Modo del sensor = RAWMODE
            SerialPort.Write(byteOut, 0, 7)

            'GET INPUT VALUES
            byteOut(0) = &H3 ' Cantidad de bytes que va a enviar 
            byteOut(1) = &H0 ' Siempre 0
            byteOut(2) = &H0 ' &H0 = se espera respuesta &H80 = no se espera respuesta
            byteOut(3) = &H7 ' Comando a enviar (GETINPUTVALUES)
            byteOut(4) = &H2 'PUERTO
            SerialPort.Write(byteOut, 0, 5)


            byteIn(0) = SerialPort.ReadByte 'Numero de bytes en la respuesta
            byteIn(1) = SerialPort.ReadByte 'Debe ser 0 para el NXT.
            For g = 2 To 1 + byteIn(0) 'Leemos el resto del mensaje.
                byteIn(g) = SerialPort.ReadByte()
            Next

            Select Case byteIn(14) 'En el byte 14 se encuentra el numero asociado al color
                Case 1 'Color Negro
                    Color.Text = "Negro"  'Texto a mostrar en el display
                    Color.ForeColor = Drawing.Color.White 'Color del texto a mostrar
                    Color.BackColor = Drawing.Color.Black 'Color de fondo del display
                Case 2 'Color Azul
                    Color.Text = "Azul"
                    Color.ForeColor = Drawing.Color.White
                    Color.BackColor = Drawing.Color.Blue
                Case 3 'Color Verde
                    Color.Text = "Verde"
                    Color.ForeColor = Drawing.Color.Black
                    Color.BackColor = Drawing.Color.LimeGreen
                Case 4 'Color Amarillo
                    Color.Text = "Amarillo"
                    Color.ForeColor = Drawing.Color.Black
                    Color.BackColor = Drawing.Color.Yellow
                Case 5 'Color Rojo
                    Color.Text = "Rojo"
                    Color.ForeColor = Drawing.Color.White
                    Color.BackColor = Drawing.Color.Red
                Case 6 'Color Blanco
                    Color.Text = "Blanco"
                    Color.ForeColor = Drawing.Color.Black
                    Color.BackColor = Drawing.Color.White

            End Select

        Catch ex As Exception

            TimerColor.Enabled = False 'En caso de error desactivamos el timer
            MsgBox("ERROR EN LA LECTURA DE COLOR")          'En caso de error mostramos este mensaje

        End Try

    End Sub


    Public Sub TimerColor_Tick(sender As Object, e As EventArgs) Handles TimerColor.Tick
        Call SensorLuz_Click(sender, e) 'Llamamos a esta funcion cada 300ms
    End Sub


    Private Sub LVerde_Click(sender As Object, e As EventArgs) Handles LVerde.Click
        Dim byteOut(6) As Byte

        Try

            'SET IMPUT MODE
            byteOut(0) = &H5 ' Cantidad de bytes que va a enviar 
            byteOut(1) = &H0 ' Siempre 0
            byteOut(2) = &H80 ' &H0 = se espera respuesta &H80 = no se espera respuesta
            byteOut(3) = &H5 ' Comando a enviar (SETINPUTMODE)
            byteOut(4) = &H2 'Puerto
            byteOut(5) = &HF 'Emitir Luz Verde
            byteOut(6) = &H0 'Modo RAWMODE
            SerialPort.Write(byteOut, 0, 7)

        Catch ex As Exception
            MsgBox("FALLO EN LA LAMPARA")
        End Try
    End Sub

    Private Sub LAzul_Click(sender As Object, e As EventArgs) Handles LAzul.Click
        Dim byteOut(6) As Byte

        Try
            'SET IMPUT MODE
            byteOut(0) = &H5 ' Cantidad de bytes que va a enviar - 2
            byteOut(1) = &H0 ' Siempre 0
            byteOut(2) = &H80 ' &H0 = se espera respuesta &H80 = no se espera respuesta
            byteOut(3) = &H5 ' Comando a enviar (SETINPUTMODE)
            byteOut(4) = &H2 'Puerto
            byteOut(5) = &H10 'Emitir luz Azul
            byteOut(6) = &H0 'RAWMODE
            SerialPort.Write(byteOut, 0, 7)

        Catch ex As Exception
            MsgBox("FALLO EN LA LAMPARA")
        End Try
    End Sub

    Private Sub LRoja_Click(sender As Object, e As EventArgs) Handles LRoja.Click
        Dim byteOut(6) As Byte
        
        Try  

            'SET IMPUT MODE
            byteOut(0) = &H5 ' Cantidad de bytes que va a enviar - 2
            byteOut(1) = &H0 ' Siempre 0
            byteOut(2) = &H80 ' &H0 = se espera respuesta &H80 = no se espera respuesta
            byteOut(3) = &H5 ' Comando a enviar (SETINPUTMODE)
            byteOut(4) = &H2 'Puerto
            byteOut(5) = &HE 'Emitir luz Roja
            byteOut(6) = &H0 'RAWMODE
            SerialPort.Write(byteOut, 0, 7)

        Catch ex As Exception
            MsgBox("FALLO EN LA LAMPARA")
        End Try
    End Sub

    Public Sub Apagarluz_Click(sender As Object, e As EventArgs) Handles Apagarluz.Click
        Dim byteOut(6) As Byte

        TimerColor.Enabled = False 'Detenemos el timer para la lectura de color
        TimerBucle.Enabled = False 'Detenemos el timer para el siguelinea

        Call Parada_Click(sender, e) 'Llamamos a la funcion de parada de mototes.

        Try
            'SET IMPUT MODE
            byteOut(0) = &H5 ' Cantidad de bytes que va a enviar - 2
            byteOut(1) = &H0 ' Siempre 0
            byteOut(2) = &H80 ' &H0 = reply expected &H80 = no reply expected
            byteOut(3) = &H5 ' Comando a enviar (GETINPUTVALUES)
            byteOut(4) = &H2 'Puerto
            byteOut(5) = &H0 'NO SENSOR, con este comando apagamos el sensor de color.
            byteOut(6) = &H0
            SerialPort.Write(byteOut, 0, 7)

            'Inicializamo el display
            Color.Text = ""
            Color.BackColor = Drawing.Color.LightSteelBlue
            Color.ForeColor = Drawing.Color.Gray

        Catch ex As Exception
            MsgBox("ERROR")
        End Try
    End Sub


    Private Sub Siguelinea_Click(sender As Object, e As EventArgs) Handles Siguelinea.Click
        Dim i, t As Integer      'Contadores
        Velocidad.Text = "40"    'Se establece una velocidad
        KeyPreview = True        'Establecemos que el formulario reciba todos los elementos

        Try
            Call SensorLuz_Click(sender, e)             'Realizamos lectura de color

            If Color.Text = "Negro" Then                'Mientras el color sea negro el robot avanza
                Do While Color.Text = "Negro"
                    Call Adelante_MouseDown(sender, e)
                    Call SensorLuz_Click(sender, e)
                Loop
                i = 0                                   'Inicializamos los contadores
                t = 0

            ElseIf Color.Text <> "Negro" Then           'Si el color no es negro se realiza un barrido hasta encontrarlo

                Do Until i = 5 Or Color.Text = "Negro"
                    Call Derecha_MouseDown(sender, e)
                    Call SensorLuz_Click(sender, e)
                    i = i + 1
                Loop

                Do Until t = 10 Or Color.Text = "Negro"
                    Call Izquierda_MouseDown(sender, e)
                    Call SensorLuz_Click(sender, e)
                    t = t + 1
                Loop

            End If

            If ModifierKeys = Keys.Control Then         'Si la tecla pulsada es CONTROL desactivamos el Siguelinea
                Call Apagarluz_Click(sender, e)
            Else                                        'Si no se para levemente el robot y se activa un timer para volver a llamar al Siguelinea
                Call Parada_Click(sender, e)
                TimerBucle.Enabled = True
            End If

        Catch ex As Exception
            MsgBox("NO SE ENCUENTRA LINEA A SEGUIR")
        End Try

    End Sub

    Private Sub TimerBucle_Tick(sender As Object, e As EventArgs) Handles TimerBucle.Tick   'Pasados 100ms se desactiva el timer y vuelve a llamarse al Siguelinea
        TimerBucle.Enabled = False
        Call Siguelinea_Click(sender, e)
    End Sub

    Private Sub TimerTacto_Tick(sender As Object, e As EventArgs) Handles TimerTacto.Tick
        Dim byteOut(6) As Byte
        Dim byteIn(17) As Byte
        Dim g As Integer = 0

        Try
            'SET INPUT MODE
            byteOut(0) = &H5 ' Cantidad de bytes que va a enviar 
            byteOut(1) = &H0 ' Siempre 0
            byteOut(2) = &H80 ' &H0 = se espera respuesta &H80 = no se espera respuesta
            byteOut(3) = &H5 ' Comando a enviar (SETINPUTMODE)
            byteOut(4) = &H1 'Puerto
            byteOut(5) = &H1 'tipo de sensor (SWITCH)
            byteOut(6) = &H20 'Modo del sensor = BOOLEAN MODE
            SerialPort.Write(byteOut, 0, 7)

            'GET INPUT VALUES
            byteOut(0) = &H3 ' Cantidad de bytes que va a enviar 
            byteOut(1) = &H0 ' Siempre 0
            byteOut(2) = &H0 ' &H0 = se espera respuesta &H80 = no se espera respuesta
            byteOut(3) = &H7 ' Comando a enviar (GETINPUTVALUES)
            byteOut(4) = &H1 'PUERTO
            SerialPort.Write(byteOut, 0, 5)

            byteIn(0) = SerialPort.ReadByte 'Numero de bytes en la respuesta
            byteIn(1) = SerialPort.ReadByte 'Debe ser 0 para el NXT.
            For g = 2 To 1 + byteIn(0) 'Leemos el resto del mensaje.
                byteIn(g) = SerialPort.ReadByte()
            Next

            If byteIn(14) = 1 Then                           'Si se pulsa
                Señal.BackColor = Drawing.Color.White       'La barra se vuelve blanca
                Call LVerde_Click(sender, e)                'Se enciende el LED Verde
            Else                                            'Si no se pulsa
                Señal.BackColor = Drawing.Color.Black       'La barra se vuelve negra
                Call Apagarluz_Click(sender, e)             'Se apaga el LED
            End If

        Catch ex As Exception
            TimerTacto.Enabled = False              'En caso de error desactivamos el timer
            MsgBox("ERROR EN LA LECTURA")          'En caso de error mostramos este mensaje
        End Try
    End Sub


    Private Sub SensorTacto_Click(sender As Object, e As EventArgs) Handles SensorTacto.Click
        TimerTacto.Enabled = True  'Activamos el timer 
    End Sub

    Private Sub ApagarTacto_Click(sender As Object, e As EventArgs) Handles ApagarTacto.Click
        TimerTacto.Enabled = False  'Desactivamos el Timer
    End Sub


    Private Sub ButtonBluetooh_Click(sender As Object, e As EventArgs) Handles ButtonBluetooh.Click
        CAT.Show()
        TimerBatery.Enabled = False
    End Sub

End Class




